==> matrix_sdk-0.5.3.tar.gz: lib/src/user.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:chopper/chopper.dart';
import 'package:kiwi/kiwi.dart';
import 'package:matrix_sdk/src/api/access_token_interceptor.dart';
import 'package:matrix_sdk/src/api/client.dart';
import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/homeserver.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/room/rooms.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:matrix_sdk/src/util/chopper.dart';
import 'package:meta/meta.dart';
import 'package:http/http.dart' as http;
import 'dart:io';

class User extends Identifiable<UserId> {
  @override
  final UserId id;

  String _name;
  String get name => _name ?? id.toString();

  User({@required this.id, String name}) : _name = name;
}

/// A user which is authenticated and can send messages, join rooms etc.
class LocalUser extends User {
  final Store _store;

  final Homeserver homeserver;
  final String accessToken;

  bool _isLoggedOut = false;
  bool get isLoggedOut => _isLoggedOut;

  /// @internal
  final String scope;

  String _syncToken;

  final ChopperClient _chopper;
  ClientService get _client => _chopper.getService();

  static Future<LocalUser> fromStore(Store store) async {
    final isOpen = await store.open();

    if (!isOpen) {
      return null;
    }

    final userId = await store.getUserId();
    if (userId == null) {
      return null;
    }

    final hs = await store.getHomeserver();
    if (hs == null) {
      return null;
    }

    final accessToken = await store.getAccessToken();
    if (accessToken == null) {
      return null;
    }

    return LocalUser(
      id: userId,
      homeserver: hs,
      accessToken: accessToken,
      store: store
    );
  }

  LocalUser(
    {@required UserId id,
    @required this.homeserver,
    @required this.accessToken,
    @required Store store})
    : _chopper = createChopper(
          uri: homeserver.uri,
          interceptors: [AccessTokenInterceptor(accessToken)]),
      rooms = Rooms(store: store),
      _store = store,
      scope = homeserver.scope + id.toString(),
      super(id: id) {
    Container().registerInstance(this, name: scope);
    Container().registerInstance(_client, name: scope);
  }

  final Rooms rooms;

  /// Invalidates the access token of the user. Makes all
  /// [LocalUser] calls unusable.
  Future<void> logout() async {
    await _client.logout();
    _isLoggedOut = true;
  }

  /// Send all unsent messages still in the [Store].
  Future<void> sendAllUnsent() async {
    await for (Room room in rooms.all()) {
      if (room is JoinedRoom) {
        await for (RoomEvent event in _store.getUnsentEvents(room)) {
          await for (final sentState in room.send(
            event.content,
            transactionId: event.unsigned?.transactionId
          )) { }
        }
      }
    }
  }

  // Sync //
  /// Syncs data with the user's [homeserver]. Use the returned [Stream] to
  /// get when the sync has been successful (`true`) or unsuccessful (`false`).
  Stream<SyncState> sync(
    {Duration retryAfter = const Duration(seconds: 30)}) async* {

    if (isLoggedOut) {
      throw StateError("The user can not be logged out");
    }

    _syncToken = await _store?.getSyncToken() ?? _syncToken;

    Response response;

    loop:
    while (true) {
      try {
        response = await _client.sync(
            since: _syncToken,
            fullState: true,
            filter: json.encode(const {
              'room': {
                'state': {
                  'lazy_load_members': true
                }
              }
            })
        );
      } on http.ClientException {
        yield SyncState(
          succeeded: false,
          failReason: SyncFailReason.connection
        );
        await Future.delayed(retryAfter);
        continue loop;
      } on SocketException {
        yield SyncState(
          succeeded: false,
          failReason: SyncFailReason.connection
        );
        await Future.delayed(retryAfter);
        continue loop;
      }

      final body = json.decode(response.body);

      // Store sync token
      _syncToken = body['next_batch'] as String;
      await _store?.setSyncToken(_syncToken);

      await _processRooms(body, body['account_data']);

      yield SyncState(succeeded: true);

      await Future.delayed(Duration(seconds: 5));
    }
  }

  Future<void> _processRooms(
      Map<String, dynamic> body,
      Map<String, dynamic> accountData) async {
    await _processJoinedRooms(body['rooms']['join'], accountData);
  }

  Future<void> _processJoinedRooms(
      Map<String, dynamic> joinedRooms,
      Map<String, dynamic> accountData) async {
    final processed = List<JoinedRoom>();

    for (final entry in joinedRooms.entries) {
      final roomId = RoomId(entry.key);
      final j = entry.value;

      Room room = await rooms[roomId];
      if (room == null) {
        room = JoinedRoom.fromJson(roomId, scope, _store, j);
      }

      await room.processSync(j);
      await room.processUserAccountData(accountData);


      processed.add(room);
    }

    // Save rooms
    await rooms.addAll(processed);
  }

  // End Sync //
}

class SyncState {
  final bool succeeded;

  /// If [succeeded] is false, this is the reason for the failure.
  final SyncFailReason failReason;

  SyncState({@required this.succeeded, this.failReason});
}

enum SyncFailReason {
  connection
}

==> matrix_sdk-0.5.4.tar.gz: lib/src/user.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:chopper/chopper.dart';
import 'package:kiwi/kiwi.dart';
import 'package:matrix_sdk/src/api/access_token_interceptor.dart';
import 'package:matrix_sdk/src/api/client.dart';
import 'package:matrix_sdk/src/event/room/room_event.dart';
import 'package:matrix_sdk/src/homeserver.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/room/rooms.dart';
import 'package:matrix_sdk/src/store.dart';
import 'package:matrix_sdk/src/util/chopper.dart';
import 'package:meta/meta.dart';
import 'package:http/http.dart' as http;
import 'dart:io';

class User extends Identifiable<UserId> {
  @override
  final UserId id;

  String _name;
  String get name => _name ?? id.toString();

  final Uri avatarUrl;

  User({@required this.id, String name, this.avatarUrl}) : _name = name;
}

/// A user which is authenticated and can send messages, join rooms etc.
class LocalUser extends User {
  final Store _store;

  final Homeserver homeserver;
  final String accessToken;

  bool _isLoggedOut = false;
  bool get isLoggedOut => _isLoggedOut;

  /// @internal
  final String scope;

  String _syncToken;

  final ChopperClient _chopper;
  ClientService get _client => _chopper.getService();

  static Future<LocalUser> fromStore(Store store) async {
    final isOpen = await store.open();

    if (!isOpen) {
      return null;
    }

    final userId = await store.getUserId();
    if (userId == null) {
      return null;
    }

    final hs = await store.getHomeserver();
    if (hs == null) {
      return null;
    }

    final accessToken = await store.getAccessToken();
    if (accessToken == null) {
      return null;
    }

    return LocalUser(
      id: userId,
      homeserver: hs,
      accessToken: accessToken,
      store: store
    );
  }

  LocalUser(
    {@required UserId id,
    @required this.homeserver,
    @required this.accessToken,
    @required Store store})
    : _chopper = createChopper(
          uri: homeserver.uri,
          interceptors: [AccessTokenInterceptor(accessToken)]),
      rooms = Rooms(store: store),
      _store = store,
      scope = homeserver.scope + id.toString(),
      super(id: id) {
    Container().registerInstance(this, name: scope);
    Container().registerInstance(_client, name: scope);
  }

  final Rooms rooms;

  /// Invalidates the access token of the user. Makes all
  /// [LocalUser] calls unusable.
  Future<void> logout() async {
    await _client.logout();
    _isLoggedOut = true;
  }

  /// Send all unsent messages still in the [Store].
  Future<void> sendAllUnsent() async {
    await for (Room room in rooms.all()) {
      if (room is JoinedRoom) {
        await for (RoomEvent event in _store.getUnsentEvents(room)) {
          await for (final sentState in room.send(
            event.content,
            transactionId: event.unsigned?.transactionId
          )) { }
        }
      }
    }
  }

  // Sync //
  /// Syncs data with the user's [homeserver]. Use the returned [Stream] to
  /// get when the sync has been successful (`true`) or unsuccessful (`false`).
  Stream<SyncState> sync(
    {Duration retryAfter = const Duration(seconds: 30)}) async* {

    if (isLoggedOut) {
      throw StateError("The user can not be logged out");
    }

    _syncToken = await _store?.getSyncToken() ?? _syncToken;

    Response response;

    loop:
    while (true) {
      try {
        response = await _client.sync(
            since: _syncToken,
            fullState: true,
            filter: json.encode(const {
              'room': {
                'state': {
                  'lazy_load_members': true
                }
              }
            })
        );
      } on http.ClientException {
        yield SyncState(
          succeeded: false,
          failReason: SyncFailReason.connection
        );
        await Future.delayed(retryAfter);
        continue loop;
      } on SocketException {
        yield SyncState(
          succeeded: false,
          failReason: SyncFailReason.connection
        );
        await Future.delayed(retryAfter);
        continue loop;
      }

      final body = json.decode(response.body);

      // Store sync token
      _syncToken = body['next_batch'] as String;
      await _store?.setSyncToken(_syncToken);

      await _processRooms(body, body['account_data']);

      yield SyncState(succeeded: true);

      await Future.delayed(Duration(seconds: 5));
    }
  }

  Future<void> _processRooms(
      Map<String, dynamic> body,
      Map<String, dynamic> accountData) async {
    await _processJoinedRooms(body['rooms']['join'], accountData);
  }

  Future<void> _processJoinedRooms(
      Map<String, dynamic> joinedRooms,
      Map<String, dynamic> accountData) async {
    final processed = List<JoinedRoom>();

    for (final entry in joinedRooms.entries) {
      final roomId = RoomId(entry.key);
      final j = entry.value;

      Room room = await rooms[roomId];
      if (room == null) {
        room = JoinedRoom.fromJson(roomId, scope, _store, j);
      }

      await room.processSync(j);
      await room.processUserAccountData(accountData);


      processed.add(room);
    }

    // Save rooms
    await rooms.addAll(processed);
  }

  // End Sync //
}

class SyncState {
  final bool succeeded;

  /// If [succeeded] is false, this is the reason for the failure.
  final SyncFailReason failReason;

  SyncState({@required this.succeeded, this.failReason});
}

enum SyncFailReason {
  connection
}
